clearvars;
%% modelling of U-Th-Pb systematics for a an inclusion that is symmetric around one axis (e.g. finite cylinder)

%this code models:
    %204Pb-diffusion only
    %206Pb,207Pb,208Pb-duffision+production
    %235U,228U,232Th-decay only
    %either t-T path of your choice or that assumed by Dodson's equation

%the code is structured as follows
    %Inputs
        %physical dimensions of the model and physical constants
        %U-Th-Pb distribution
        %t-T path to model
    %Calculations
        %calculation of inital 235,238U, 232Th and 204,206,207,208Pb profiles
        %modelling of the t-T path
        %calculation of bulk-grain dates
    %Output
        %create output txt file (the structure of the output txt file is described in this section)

%important notes
    %input U and Pb concetration maps must have exactly the same size in points, and Nz and Nr variables must match this size 
    %207Pb-corrected dates are calculated correctrly only when they exceed 1 Ma


%% physical dimensions of the model and physical constants

%model resolution
Nr=58; % number of  nodes in r direction
Nz=114; % number of nodes in r direction
dr=0.473; %distance beween the nodes in r direction
dz=dr; %distance beween the nodes in z direction (equal to that in r direction)
Lr = dr*(Nr-1); %radius of the model
Lz = dz*(Nz-1); %height of the model
r=0:dr:Lr; %distance from the centre of the model
z=0:dz:Lz; z=z'; %distance from the top of the model

%physical constants used in the model
R = 8.3144598; %gas constant in J*mol^-1*K^-1
lambda238 = 4.91933E-18; %238U decay constant in sec^-1
lambda235 = 3.1221E-17; %235U decay constant in sec^-1
lambda232 = 1.56438E-18; %232Th decay constant sec^-1
presentU238U235=137.88125; %present-day ratio between the U isotopes
year=31556925.445; %year duration  in sec
KdPb=5;  %partiotion coefficient in a form Host/Inclusion

% diffusion properties of the inclusion
% radial
InclDPbr0 = 12700;  %frequency factor in micron^2*sec^-1
InclEactPbr = 228469.8063;  %activation energy in J*mol^-1
% axial
InclDPbz0 = 12700; %frequency factor in micron^2*sec^-1
InclEactPbz = 228469.8063; %activation energy in J*mol^-1

% diffusion properties of the host-mineral
% radial
HostDPbr0 = 14200000; %frequency factor in micron^2*sec^-1
HostEactPbr = 305000; %activation energy in J*mol^-1
% axial
HostDPbz0 = 14200000; %frequency factor in micron^2*sec^-1
HostEactPbz = 305000; %activation energy in J*mol^-1





%generate necessary variables 
Mineral=zeros(Nz,Nr); %indicator of which mineral is in the node (0 for Host and 1 for Inclusion)
MineralIntervalsR=zeros(Nz,Nr-1); %indicator of which mineral is between the nodes in r direction (0 for Host and 1 for Inclusion)
MineralIntervalsZ=zeros(Nz-1,Nr); %indicator of which mineral is between the nodes in z direction (0 for Host and 1 for Inclusion)
Pb204=zeros(Nz,Nr); Pb206=zeros(Nz,Nr); Pb207=zeros(Nz,Nr); Pb208=zeros(Nz,Nr); U238=zeros(Nz,Nr); U235=zeros(Nz,Nr); Th232=zeros(Nz,Nr); %concentrations
ratiosPb206Pb204=zeros(Nz,Nr); ratiosPb207Pb204=zeros(Nz,Nr); ratiosPb208Pb204=zeros(Nz,Nr); ratiosPb206U238=zeros(Nz,Nr); ratiosPb207U235=zeros(Nz,Nr); ratiosPb208Th232=zeros(Nz,Nr); %ratios
agesPb206U238=zeros(Nz,Nr); agesPb207U235=zeros(Nz,Nr); agesPb208Th232=zeros(Nz,Nr); agesPb207Pb206=zeros(Nz,Nr); %ages
DPbrIntervals=zeros(Nz,Nr-1); %diffusivity between the nodes in r direction
DPbzIntervals=zeros(Nz-1,Nr); %diffusivity between the nodes in z direction
KdPbNodes=ones(Nz,Nr); %Kd in the nodes
KdPbIntervalsR=ones(Nz,Nr-1); %Kd between the nodes in r direction
KdPbIntervalsZ=ones(Nz-1,Nr); %Kd between the nodes in r direction
L=spalloc(Nr*Nz,Nr*Nz,((Nr-2)*(Nz-2)*5+Nz+(Nr-1)*2+(Nz-2)*4) ); R204=zeros(Nr*Nz,1); R206=zeros(Nr*Nz,1); R207=zeros(Nr*Nz,1); R208=zeros(Nr*Nz,1);

%% U-Th-Pb distribution

%importing 204Pb and 238U maps
Pbinput=im2double(imread('inputPb.png')); Pbinput=Pbinput.*255; %importing 204Pb concentration map (grey scale .png image) 
Uinput=im2double(imread('inputU.png')); Uinput=Uinput.*255; %importing 238U concentration map (grey scale .png image)
Thinput=im2double(imread('inputTh.png')); Thinput=Thinput.*255; %importing 238U concentration map (grey scale .png image)

%define the boundaries of the inclusion
Mineral(Uinput>0)=1; %rule to identify where is the inclusion

%obtain inclusion radius at half-width of the model
for i=1:1:Nr-1
    if (Mineral(round(Nz/2),i)==1)
        LrIncl=r(i);
    end
end

%define the initial 204Pb concentrations in the inclusion and in the host
scalePbinput=1; %scaling factor the input map (if needed)
Pb204=Pbinput.*scalePbinput; %obtain 204Pb concentration matrix that is used in calculations

%define the initial Pb isotopic composition
initialPb206Pb204=18.21; %206Pb/204Pb
initialPb207Pb204=16.03; %207Pb/204Pb
initialPb208Pb204=36.00; %208Pb/204Pb

%define the initial 238U concentrations in the inclusion and in the host
scaleUinput=1; %scaling factor the input map (if needed)
U238=Uinput.*scaleUinput; %obtain 238U concentration matrix that is used in calculations

%define the initial 232Th concentrations in the inclusion and in the host
scaleThinput=1; %scaling factor the input map (if needed)
Th232=Thinput.*scaleThinput; %obtain 232Th concentration matrix that is used in calculations

%% Create time-temperature path
%two options are available - t-T path assumed by Dodson's equation and custom t-T path
%use comment sign to deactivate one of the options

% %option 1 - t-T path assumed by Dodson's equation for a specified cooling rate for a sphere
% CR=1; %specify the cooling rate in degrees per m.y. (at the closure temperature)
% dtinflections=0.1; %specify the time gap between any two points in the t-T path
% inflectionsCRlength=10000; %specify the number of points in the t-T path
% %12 lines below calculate the t-T path
% Tc=0; 
% while ( Tc < ((InclEactPbr/R)/(log( 55*R*(Tc+273)^2*InclDPbr0/LrIncl^2/InclEactPbr/CR*1000000*year  ) ) - 273  )  )
%       Tc=Tc+0.01;
% end
% kinflections=(1/(Tc+(-CR)/1000000)-1/(Tc-(-CR)/1000000))/(2/1000000);
% binflections=1/1000;
% timeinflections=0;
% for inflectionssteps=1:1:inflectionsCRlength
%     inflections(1,inflectionssteps)=timeinflections;
%     inflections(2,inflectionssteps)=1/(timeinflections*kinflections+binflections);
%     timeinflections=timeinflections+dtinflections;
% end

% %option 2 - custom t-T path
% inflections=[0  , 0.01,  40,  75, 170;...  %time in m.y. (0 - time of crystallisation)
%              800,  400, 400, 100,  0];  %Temperatures in Celcium degrees

%t-T path resolution
tTpointsperinterval=1; %integer that specifies the number of additional points between any two points in the "inflections" variable (calculated automatically by linear interpolation)





%calculate the t-T path that is actually used for modelling
sizeofinflections=size(inflections); % need to determine an interval t-T path within which dt and T are calculated
stepscounter=1; % counts number of iterations to write the newly obtained dt and T in a new column
tottime=0; % sums up all dt to det t-T path
T = 0; % Temperature Celcium degrees
for interval=1:1:(sizeofinflections(2)-1)
    if (inflections(1,interval+1)>inflections(1,interval))   
        k=(inflections(2,interval+1)-inflections(2,interval))/((inflections(1,interval+1)-inflections(1,interval))*1000000*year);
        b=inflections(2,interval)-k*inflections(1,interval)*1000000*year;
        stepscounter2=1;
        timedif=0;
        while (tottime<(inflections(1,interval+1)*1000000*year))
            dt=(inflections(1,interval+1)-inflections(1,interval))*1000000*year/tTpointsperinterval;

                if (k<=0)
                    T=(tottime+dt/2)*k+b;                
                else
                    T=(inflections(1,interval+1)*1000000*year-timedif-dt/2)*k+b;                
                end           

            if (k<=0)
                CoolingCurve(1,stepscounter)=dt;
                CoolingCurve(2,stepscounter)=T;
                CoolingCurve(3,stepscounter)=tottime/1000000/year;
                CoolingCurve(4,stepscounter)=inflections(1,sizeofinflections(2))-tottime/1000000/year;
                stepscounter=stepscounter+1;
            else
                HeatingInterval(1,stepscounter2)=dt;
                HeatingInterval(2,stepscounter2)=T;
                timedif=timedif+dt;
                HeatingInterval(3,stepscounter2)=timedif/1000000/year;
                stepscounter2=stepscounter2+1;
            end
            tottime=tottime+dt;
        end 

        % rewrite heating interval into cooling curve in the right order 
        if (k>0)
            stepscounteri=stepscounter;
            while (stepscounter <=(stepscounter2-2+stepscounteri) )
                CoolingCurve(1,stepscounter)=HeatingInterval(1,(stepscounter2-2+stepscounteri-stepscounter+1));
                CoolingCurve(2,stepscounter)=HeatingInterval(2,(stepscounter2-2+stepscounteri-stepscounter+1));
                CoolingCurve(3,stepscounter)=inflections(1,interval)+(HeatingInterval(3,(stepscounter2-1))-HeatingInterval(3,(stepscounter2-2+stepscounteri-stepscounter+1)));
                CoolingCurve(4,stepscounter)=inflections(1,sizeofinflections(2))-(inflections(1,interval)+(HeatingInterval(3,(stepscounter2-1))-HeatingInterval(3,(stepscounter2-2+stepscounteri-stepscounter+1))));
                stepscounter=stepscounter+1;
            end
            clearvars HeatingInterval;
        end
    else
    end
end
sizeofCoolingCurve=size(CoolingCurve);
Nt=sizeofCoolingCurve(2);
TimeSum=0;

%% diffusion/decay modelling

%preprocesseing (obtain missing variables, calclulate initial 235,238U and 232Th)
for j=1:1:Nr-1   % j - number of a column
    for i=1:1:Nz  % i - number of a row
        if (Mineral(i,j)==1 && Mineral(i,j+1)==1) 
            MineralIntervalsR(i,j) = 1;
        else
            MineralIntervalsR(i,j) = 0;
        end
    end
end
for j=1:1:Nr   % j - number of a column
    for i=1:1:Nz-1  % i - number of a row
        if (Mineral(i,j)==1 && Mineral(i+1,j)==1) 
            MineralIntervalsZ(i,j) = 1;
        else
            MineralIntervalsZ(i,j) = 0;
        end
    end
end
KdPbNodes(Mineral==1)=KdPb;
KdPbIntervalsR(MineralIntervalsR==1)=KdPb;
KdPbIntervalsZ(MineralIntervalsZ==1)=KdPb;

for j=1:1:Nr
    for i=1:1:Nz
       Pb206(i,j)=Pb204(i,j)*initialPb206Pb204;
       Pb207(i,j)=Pb204(i,j)*initialPb207Pb204;
       Pb208(i,j)=Pb204(i,j)*initialPb208Pb204;
       U235(i,j)=U238(i,j)/presentU238U235;
       U238(i,j)=U238(i,j)*exp(lambda238*(inflections(1,sizeofinflections(2))*1000000*year));
       U235(i,j)=U235(i,j)*exp(lambda235*(inflections(1,sizeofinflections(2))*1000000*year));
       Th232(i,j)=Th232(i,j)*exp(lambda232*(inflections(1,sizeofinflections(2))*1000000*year));
    end
end

% diffusion calculation
for it=1:1:Nt
    T=CoolingCurve(2,it);
    dt=CoolingCurve(1,it);
    
    % Diffusivities update
    DPbrIntervals(MineralIntervalsR==1)=InclDPbr0 * exp(-(InclEactPbr)/R/(T+273));
    DPbrIntervals(MineralIntervalsR==0)=HostDPbr0 * exp(-(HostEactPbr)/R/(T+273));
    
    DPbzIntervals(MineralIntervalsZ==1)=InclDPbz0 * exp(-(InclEactPbz)/R/(T+273));
    DPbzIntervals(MineralIntervalsZ==0)=HostDPbz0 * exp(-(HostEactPbz)/R/(T+273));
        
    % Compose matrix
    for j=1:1:Nr
        for i=1:1:Nz
            EqInd=Nz*(j-1)+i; %index of the equation
            
            % Topmost boundary
            if(i==1 || i==Nz || j==Nr)
                
                L(EqInd,EqInd)=1;%i,j

                R204(EqInd,1)=Pb204(i,j);
                R206(EqInd,1)=Pb206(i,j)+U238(i,j)*(1-exp(-lambda238*dt));
                R207(EqInd,1)=Pb207(i,j)+U235(i,j)*(1-exp(-lambda235*dt));
                R208(EqInd,1)=Pb208(i,j)+Th232(i,j)*(1-exp(-lambda232*dt));
                
            elseif(j==1)
                CoefA=(2*dt*DPbrIntervals(i,j)*KdPbNodes(i,j+1))/(KdPbIntervalsR(i,j)*(dr^2));
                CoefE=(dt*DPbzIntervals(i,j)*KdPbNodes(i+1,j))/(2*KdPbIntervalsZ(i,j)*(dz^2));
                CoefF=(dt*DPbzIntervals(i-1,j)*KdPbNodes(i-1,j))/(2*KdPbIntervalsZ(i-1,j)*(dz^2));
                CoefG=dt*KdPbNodes(i,j)*((2*DPbrIntervals(i,j))/(KdPbIntervalsR(i,j)*(dr^2))+DPbzIntervals(i,j)/(2*KdPbIntervalsZ(i,j)*(dz^2))+DPbzIntervals(i-1,j)/(2*KdPbIntervalsZ(i-1,j)*(dz^2)));
                
                L(EqInd,EqInd-1)=-CoefF;%i-1,j
                L(EqInd,EqInd)=1+CoefG;%i,j
                L(EqInd,EqInd+Nz)=-CoefA;%i,j+1
                L(EqInd,EqInd+1)=-CoefE;%i+1,j

                R204(EqInd,1)=Pb204(i,j+1)*CoefA+Pb204(i+1,j)*CoefE+Pb204(i-1,j)*CoefF+Pb204(i,j)*(1-CoefG);
                R206(EqInd,1)=Pb206(i,j+1)*CoefA+Pb206(i+1,j)*CoefE+Pb206(i-1,j)*CoefF+Pb206(i,j)*(1-CoefG)+U238(i,j)*(1-exp(-lambda238*dt));
                R207(EqInd,1)=Pb207(i,j+1)*CoefA+Pb207(i+1,j)*CoefE+Pb207(i-1,j)*CoefF+Pb207(i,j)*(1-CoefG)+U235(i,j)*(1-exp(-lambda235*dt));
                R208(EqInd,1)=Pb208(i,j+1)*CoefA+Pb208(i+1,j)*CoefE+Pb208(i-1,j)*CoefF+Pb208(i,j)*(1-CoefG)+Th232(i,j)*(1-exp(-lambda232*dt));
                
                
            else
                CoefA=(dt*(r(j)/2+r(j+1)/2)*DPbrIntervals(i,j)*KdPbNodes(i,j+1))/( 2*r(j)*KdPbIntervalsR(i,j)*(dr^2));
                CoefB=(dt*(r(j)/2+r(j-1)/2)*DPbrIntervals(i,j-1)*KdPbNodes(i,j-1))/( 2*r(j)*KdPbIntervalsR(i,j-1)*(dr^2));
                CoefE=(dt*DPbzIntervals(i,j)*KdPbNodes(i+1,j))/(2*KdPbIntervalsZ(i,j)*(dz^2));
                CoefF=(dt*DPbzIntervals(i-1,j)*KdPbNodes(i-1,j))/(2*KdPbIntervalsZ(i-1,j)*(dz^2));
                CoefG=dt*KdPbNodes(i,j)/2*(  (r(j)/2+r(j+1)/2)*DPbrIntervals(i,j)/(r(j)*KdPbIntervalsR(i,j)*(dr^2))+...
                    (r(j)/2+r(j-1)/2)*DPbrIntervals(i,j-1)/(r(j)*KdPbIntervalsR(i,j-1)*(dr^2))+...
                    DPbzIntervals(i,j)/(KdPbIntervalsZ(i,j)*(dz^2))+...
                    DPbzIntervals(i-1,j)/(KdPbIntervalsZ(i-1,j)*(dz^2)) );
                
                L(EqInd,EqInd-1)=-CoefF;%i-1,j    -f
                L(EqInd,EqInd-Nz)=-CoefB;%i,j-1   -b
                L(EqInd,EqInd)=1+CoefG;%i,j       (1+g)
                L(EqInd,EqInd+Nz)=-CoefA;%i,j+1   -a
                L(EqInd,EqInd+1)=-CoefE;%i+1,j    -e
                
                R204(EqInd,1)=Pb204(i,j+1)*CoefA+Pb204(i,j-1)*CoefB+Pb204(i+1,j)*CoefE+Pb204(i-1,j)*CoefF+Pb204(i,j)*(1-CoefG);
                R206(EqInd,1)=Pb206(i,j+1)*CoefA+Pb206(i,j-1)*CoefB+Pb206(i+1,j)*CoefE+Pb206(i-1,j)*CoefF+Pb206(i,j)*(1-CoefG)+U238(i,j)*(1-exp(-lambda238*dt));
                R207(EqInd,1)=Pb207(i,j+1)*CoefA+Pb207(i,j-1)*CoefB+Pb207(i+1,j)*CoefE+Pb207(i-1,j)*CoefF+Pb207(i,j)*(1-CoefG)+U235(i,j)*(1-exp(-lambda235*dt));
                R208(EqInd,1)=Pb208(i,j+1)*CoefA+Pb208(i,j-1)*CoefB+Pb208(i+1,j)*CoefE+Pb208(i-1,j)*CoefF+Pb208(i,j)*(1-CoefG)+Th232(i,j)*(1-exp(-lambda232*dt));
                
            end            
        end
    end
    
    S204=L\R204;
    S206=L\R206;
    S207=L\R207;
    S208=L\R208;
        
    U238=U238.*exp(-lambda238*dt);
    U235=U235.*exp(-lambda235*dt);
    Th232=Th232.*exp(-lambda232*dt);
    
    % updating fields
    for j=1:1:Nr
        for i=1:1:Nz
            EqInd=Nz*(j-1)+i; %index of the equation
            
            Pb204(i,j)=S204(EqInd,1);
            Pb206(i,j)=S206(EqInd,1);
            Pb207(i,j)=S207(EqInd,1);
            Pb208(i,j)=S208(EqInd,1);
            
        end
    end
    
    TimeSum=TimeSum+dt/year/1000000;
end

% integrate isotope quantities and calculate in situ dates
InclSumPb204=0;
InclSumPb206=0;
InclSumPb207=0;
InclSumPb208=0;
InclSumU238=0;
InclSumU235=0;
InclSumTh232=0;

for j=1:1:Nr
    for i=1:1:Nz
        ratiosPb206Pb204(i,j)=Pb206(i,j)/Pb204(i,j);
        ratiosPb207Pb204(i,j)=Pb207(i,j)/Pb204(i,j);
        ratiosPb208Pb204(i,j)=Pb208(i,j)/Pb204(i,j);
        
        if (U238(i,j)>0)
            ratiosPb206U238(i,j)=Pb206(i,j)/U238(i,j);
            ratiosPb207U235(i,j)=Pb207(i,j)/U235(i,j);
            ratiosPb208Th232(i,j)=Pb208(i,j)/Th232(i,j);
                        
            agesPb206U238(i,j)=log((Pb206(i,j)/Pb204(i,j)-initialPb206Pb204)/(U238(i,j)/Pb204(i,j))+1)/lambda238/year/1000000;
            agesPb207U235(i,j)=log((Pb207(i,j)/Pb204(i,j)-initialPb207Pb204)/(U235(i,j)/Pb204(i,j))+1)/lambda235/year/1000000;
            agesPb208Th232(i,j)=log((Pb208(i,j)/Pb204(i,j)-initialPb208Pb204)/(Th232(i,j)/Pb204(i,j))+1)/lambda232/year/1000000;
        
            agesPb207Pb206(i,j)=1;
            while ( 0.0001 < sqrt((...
                    ( (Pb207(i,j)/Pb206(i,j)- initialPb207Pb204/initialPb206Pb204)/(U238(i,j)/Pb206(i,j)) ) * ...
                    ( 1/(exp(lambda238*year*1000000*agesPb207Pb206(i,j))-1) )+...
                    ( Pb207(i,j)/Pb206(i,j) -  U238(i,j)/Pb206(i,j)*( (Pb207(i,j)/Pb206(i,j)- initialPb207Pb204/initialPb206Pb204)/(U238(i,j)/Pb206(i,j)) ) )-...
                    ( exp(lambda235*year*1000000*agesPb207Pb206(i,j))-1 ) / (exp(lambda238*year*1000000*agesPb207Pb206(i,j))-1)/presentU238U235   )^2)   )
                
                agesPb207Pb206(i,j)=agesPb207Pb206(i,j)+ sqrt((...
                    ( (Pb207(i,j)/Pb206(i,j)- initialPb207Pb204/initialPb206Pb204)/(U238(i,j)/Pb206(i,j)) ) * ...
                    ( 1/(exp(lambda238*year*1000000*agesPb207Pb206(i,j))-1) )+...
                    ( Pb207(i,j)/Pb206(i,j) -  U238(i,j)/Pb206(i,j)*( (Pb207(i,j)/Pb206(i,j)- initialPb207Pb204/initialPb206Pb204)/(U238(i,j)/Pb206(i,j)) ) )-...
                    ( exp(lambda235*year*1000000*agesPb207Pb206(i,j))-1 ) / (exp(lambda238*year*1000000*agesPb207Pb206(i,j))-1)/presentU238U235   )^2)/2;
            end
        else
            ratiosPb206U238(i,j)=NaN;
            ratiosPb207U235(i,j)=NaN;
            ratiosPb208Th232(i,j)=NaN;
            
            agesPb206U238(i,j)=NaN;
            agesPb207U235(i,j)=NaN;
            agesPb208Th232(i,j)=NaN;
            agesPb207Pb206(i,j)=NaN;
        end
        if (Mineral(i,j)==1 && Mineral(i+1,j)==1 && Mineral(i-1,j)==1 && Mineral(i,j+1)==1 && j==1)
            InclSumPb204=InclSumPb204+Pb204(i,j)*pi*dz*(dr^2)/4;
            InclSumPb206=InclSumPb206+Pb206(i,j)*pi*dz*(dr^2)/4;
            InclSumPb207=InclSumPb207+Pb207(i,j)*pi*dz*(dr^2)/4;
            InclSumPb208=InclSumPb208+Pb208(i,j)*pi*dz*(dr^2)/4;
            InclSumU238=InclSumU238+U238(i,j)*pi*dz*(dr^2)/4;
            InclSumU235=InclSumU235+U235(i,j)*pi*dz*(dr^2)/4;
            InclSumTh232=InclSumTh232+Th232(i,j)*pi*dz*(dr^2)/4;
        elseif(Mineral(i,j)==1 && Mineral(i+1,j)==1 && Mineral(i-1,j)==1 && Mineral(i,j+1)==1 && Mineral(i,j-1)==1 && j>1)
            InclSumPb204=InclSumPb204+Pb204(i,j)*pi*dz*dr*r(j)*2;
            InclSumPb206=InclSumPb206+Pb206(i,j)*pi*dz*dr*r(j)*2;
            InclSumPb207=InclSumPb207+Pb207(i,j)*pi*dz*dr*r(j)*2;
            InclSumPb208=InclSumPb208+Pb208(i,j)*pi*dz*dr*r(j)*2;
            InclSumU238=InclSumU238+U238(i,j)*pi*dz*dr*r(j)*2;
            InclSumU235=InclSumU235+U235(i,j)*pi*dz*dr*r(j)*2;
            InclSumTh232=InclSumTh232+Th232(i,j)*pi*dz*dr*r(j)*2;
        end
    end
end

WGagePb206U238=log((InclSumPb206/InclSumPb204-initialPb206Pb204)/(InclSumU238/InclSumPb204)+1)/lambda238/year/1000000;
WGagePb207U235=log((InclSumPb207/InclSumPb204-initialPb207Pb204)/(InclSumU235/InclSumPb204)+1)/lambda235/year/1000000;
WGagePb208Th232=log((InclSumPb208/InclSumPb204-initialPb208Pb204)/(InclSumTh232/InclSumPb204)+1)/lambda232/year/1000000;

WGagePb207Pb206=1;
while ( 0.0001 < sqrt((...
        ( (InclSumPb207/InclSumPb206- initialPb207Pb204/initialPb206Pb204)/(InclSumU238/InclSumPb206) ) * ...
        ( 1/(exp(lambda238*year*1000000*WGagePb207Pb206)-1) )+...
        ( InclSumPb207/InclSumPb206 -  InclSumU238/InclSumPb206*( (InclSumPb207/InclSumPb206- initialPb207Pb204/initialPb206Pb204)/(InclSumU238/InclSumPb206) ) )-...
        ( exp(lambda235*year*1000000*WGagePb207Pb206)-1 ) / (exp(lambda238*year*1000000*WGagePb207Pb206)-1)/presentU238U235   )^2)   )
                
                WGagePb207Pb206=WGagePb207Pb206+ sqrt((...
        ( (InclSumPb207/InclSumPb206- initialPb207Pb204/initialPb206Pb204)/(InclSumU238/InclSumPb206) ) * ...
        ( 1/(exp(lambda238*year*1000000*WGagePb207Pb206)-1) )+...
        ( InclSumPb207/InclSumPb206 -  InclSumU238/InclSumPb206*( (InclSumPb207/InclSumPb206- initialPb207Pb204/initialPb206Pb204)/(InclSumU238/InclSumPb206) ) )-...
        ( exp(lambda235*year*1000000*WGagePb207Pb206)-1 ) / (exp(lambda238*year*1000000*WGagePb207Pb206)-1)/presentU238U235   )^2)/2;
 end

%calculate closure temperatures
if exist('CR','var')    
     for TcFinder=1:1:sizeofCoolingCurve(2)
         if (WGagePb206U238<=CoolingCurve(4,TcFinder) )
         Tc206238=CoolingCurve(2,TcFinder);
         end

         if (WGagePb207U235<=CoolingCurve(4,TcFinder) )
         Tc207235=CoolingCurve(2,TcFinder);
         end

         if (WGagePb207Pb206<=CoolingCurve(4,TcFinder) )
         Tc207206=CoolingCurve(2,TcFinder);
         end

         if (WGagePb208Th232<=CoolingCurve(4,TcFinder) )
         Tc208232=CoolingCurve(2,TcFinder);
         end    
     end
else
    Tc=NaN;
    Tc206238=NaN;
    Tc207235=NaN;
    Tc207206=NaN;
    Tc208232=NaN;
end

%% Output

%.mat file
save 'results.mat';

%.txt files
%summary.txt (whole grain dates and profiles across the half-height of the model)
%go through the comments to see the structure of the summary.txt file 
%(lines in the output file appear in the same order)
for i=1:1:Nr
    if (i==1) %the first column in the output table
        Exportdatasheet(1,i)=LrIncl; %radius of the inclusion
        Exportdatasheet(2,i)=r(i); %coodrinate of any given node
        Exportdatasheet(3,i)=Pb204(round(Nz/2),i); %204Pb concentration in any given node
        Exportdatasheet(4,i)=Pb206(round(Nz/2),i); %206Pb concentration in any given node
        Exportdatasheet(5,i)=Pb207(round(Nz/2),i); %207Pb concentration in any given node
        Exportdatasheet(6,i)=Pb208(round(Nz/2),i); %208Pb concentration in any given node
        Exportdatasheet(7,i)=U238(round(Nz/2),i); %238U concentration in any given node
        Exportdatasheet(8,i)=U235(round(Nz/2),i); %235U concentration in any given node
        Exportdatasheet(9,i)=Th232(round(Nz/2),i); %232 Th concentration in any given node
        Exportdatasheet(10,i)=agesPb206U238(round(Nz/2),i); %206Pb/238U date in any given node
        Exportdatasheet(11,i)=agesPb207U235(round(Nz/2),i); %207Pb/235U date in any given node
        Exportdatasheet(12,i)=agesPb207Pb206(round(Nz/2),i); %207Pb-corrected date in any given node
        Exportdatasheet(13,i)=agesPb208Th232(round(Nz/2),i); %208Pb/232Th date in any given node
        Exportdatasheet(14,i)=WGagePb206U238; %bulk grain 206Pb/238U date
        Exportdatasheet(15,i)=WGagePb207U235; %bulk grain 207Pb/2358U date
        Exportdatasheet(16,i)=WGagePb207Pb206; %bulk grain 207Pb-corrected date
        Exportdatasheet(17,i)=WGagePb208Th232; %bulk grain 208Pb/232Th date
        Exportdatasheet(18,i)=Tc; %Dodson's closure temperature
        Exportdatasheet(19,i)=Tc206238; %closure temperature calculated using bulk grain 206Pb/238U date
        Exportdatasheet(20,i)=Tc207235; %closure temperature calculated using bulk grain 207Pb/235U date
        Exportdatasheet(21,i)=Tc207206; %closure temperature calculated using bulk grain 207Pb-corrected date
        Exportdatasheet(22,i)=Tc208232; %closure temperature calculated using bulk grain 208Pb/232Th date
    else %other columns in the output table
        Exportdatasheet(2,i)=r(i); %coodrinate of any given node
        Exportdatasheet(3,i)=Pb204(round(Nz/2),i); %204Pb concentration in any given node
        Exportdatasheet(4,i)=Pb206(round(Nz/2),i); %206Pb concentration in any given node
        Exportdatasheet(5,i)=Pb207(round(Nz/2),i); %207Pb concentration in any given node
        Exportdatasheet(6,i)=Pb208(round(Nz/2),i); %208Pb concentration in any given node
        Exportdatasheet(7,i)=U238(round(Nz/2),i); %238U concentration in any given node
        Exportdatasheet(8,i)=U235(round(Nz/2),i); %235U concentration in any given node
        Exportdatasheet(9,i)=Th232(round(Nz/2),i); %232 Th concentration in any given node
        Exportdatasheet(10,i)=agesPb206U238(round(Nz/2),i); %206Pb/238U date in any given node
        Exportdatasheet(11,i)=agesPb207U235(round(Nz/2),i); %207Pb/235U date in any given node
        Exportdatasheet(12,i)=agesPb207Pb206(round(Nz/2),i); %207Pb-corrected date in any given node
        Exportdatasheet(13,i)=agesPb208Th232(round(Nz/2),i); %208Pb/232Th date in any given node
    end
end
dlmwrite('output_summary.txt', Exportdatasheet, 'delimiter', '\t');
dlmwrite('output_Pb204.txt', Pb204, 'delimiter', '\t'); %204Pb concentrations
dlmwrite('output_Pb206.txt', Pb206, 'delimiter', '\t'); %206Pb concentrations
dlmwrite('output_Pb207.txt', Pb207, 'delimiter', '\t'); %207Pb concentrations
dlmwrite('output_Pb208.txt', Pb208, 'delimiter', '\t'); %208Pb concentrations
dlmwrite('output_U238.txt', U238, 'delimiter', '\t'); %238U concentrations
dlmwrite('output_U235.txt', U235, 'delimiter', '\t'); %235U concentrations
dlmwrite('output_Th232.txt', Th232, 'delimiter', '\t'); %232Th concentrations
dlmwrite('output_datesPb206U238.txt', agesPb206U238, 'delimiter', '\t'); %Pb206/U238 dates concentrations
dlmwrite('output_datesPb207Pb206.txt', agesPb207Pb206, 'delimiter', '\t'); %Pb206/U238 dates concentrations
dlmwrite('output_datesPb207U235.txt', agesPb207U235, 'delimiter', '\t'); %Pb206/U238 dates concentrations
dlmwrite('output_datesPb208Th232.txt', agesPb208Th232, 'delimiter', '\t'); %Pb206/U238 dates concentrations

%figures
%profiles across the half-width of the model
figure(1);clf;

subplot(3,2,1);
plot(r,Pb204(round(Nz/2),:), ...
    r,Pb206(round(Nz/2),:), ...
    r,Pb207(round(Nz/2),:), ...
    r,Pb208(round(Nz/2),:),'LineWidth',2);
xlabel('r (\mum)');
ylabel('^2^0^8Pb (ppm)')
title('Pb concentrations');
legend('^2^0^4Pb','^2^0^6Pb','^2^0^7Pb','^2^0^8Pb');

subplot(3,2,2);
plot(r,U238(round(Nz/2),:), ...
    r,Th232(round(Nz/2),:),'LineWidth',2);
xlabel('r (\mum)');
ylabel('^2^3^8U (ppm)')
title('U, Th concentrations');
legend('^2^3^8U','^2^3^2Th');

subplot(3,2,3);
plot(-inflections(1,:)+(max(inflections(1,:))),inflections(2,:), 'LineWidth',2);
set(gca,'Xdir','reverse');
title('t-T path');
xlabel('Time (Ma)');
ylabel('Temperature (\circC)');

subplot(3,2,4);
plot(r,agesPb206U238(round(Nz/2),:), ...
    r,agesPb207U235(round(Nz/2),:), ...
    r,agesPb207Pb206(round(Nz/2),:), ...
    r,agesPb208Th232(round(Nz/2),:),'LineWidth',2);
ylim([min([min(agesPb206U238(round(Nz/2),1:round(LrIncl/dr)-1)) min(agesPb207U235(round(Nz/2),1:round(LrIncl/dr)-1)) min(agesPb207Pb206(round(Nz/2),1:round(LrIncl/dr)-1)) min(agesPb208Th232(round(Nz/2),1:round(LrIncl/dr)-1))])*0.9 max([max(agesPb206U238(round(Nz/2),1:round(LrIncl/dr)-1)) max(agesPb207U235(round(Nz/2),1:round(LrIncl/dr)-1)) max(agesPb207Pb206(round(Nz/2),1:round(LrIncl/dr)-1)) max(agesPb208Th232(round(Nz/2),1:round(LrIncl/dr)-1))])*1.1])
title('Age comparison along the R axis');
xlabel('r (\mum)');
ylabel('Date (Ma)');
legend('^2^0^6Pb/^2^3^8U date','^2^0^7Pb/^2^3^5U date','^2^0^7Pb-corrected date','^2^0^8Pb/^2^3^2Th date');

subplot(3,2,5);
plot(r,ratiosPb206Pb204(round(Nz/2),:), ...
    r,ratiosPb207Pb204(round(Nz/2),:), ...
    r,ratiosPb208Pb204(round(Nz/2),:), 'LineWidth',2);
title('Lead isotope ratios along the R axis');
xlabel('r (\mum)');
ylabel('ratio');
legend('^2^0^6Pb/^2^0^4Pb','^2^0^7Pb/^2^0^4Pb','^2^0^8Pb/^2^0^4Pb');

subplot(3,2,6);
plot(r,ratiosPb206U238(round(Nz/2),:), ...
    r,ratiosPb207U235(round(Nz/2),:), ...
    r,ratiosPb208Th232(round(Nz/2),:), 'LineWidth',2);
title('Daughter/parent ratios along the R axis');
xlabel('r (\mum)');
ylabel('ratio');
legend('^2^0^6Pb/^2^3^8U','^2^0^7Pb/^2^3^5U','^2^0^8Pb/^2^3^2Th');

print('output_profiles','-dpdf','-fillpage')

%element concentration maps
figure(1);clf;
fig=figure(1);
fig.Units = 'normalized';
fig.Position = [0 0 1 1];
subplot(2,3,1);
pcolor(r,z,Pb204);
colormap(parula);
colorbar;
shading interp;
axis ij image;
xlabel('r (\mum)');
ylabel('z (\mum)');
colorbarlabel = colorbar;
ylabel(colorbarlabel, '^2^0^4Pb (ppm)')
title('^2^0^4Pb');

subplot(2,3,2);
pcolor(r,z,Pb206);
colormap(parula);
colorbar;
shading interp;
axis ij image;
xlabel('r (\mum)');
ylabel('z (\mum)');
colorbarlabel = colorbar;
ylabel(colorbarlabel, '^2^0^6Pb (ppm)')
title('^2^0^6Pb');

subplot(2,3,3);
pcolor(r,z,Pb207);
colormap(parula);
colorbar;
shading interp;
axis ij image;
xlabel('r (\mum)');
ylabel('z (\mum)');
colorbarlabel = colorbar;
ylabel(colorbarlabel, '^2^0^7Pb (ppm)')
title('^2^0^7Pb');

subplot(2,3,4);
pcolor(r,z,Pb208);
colormap(parula);
colorbar;
shading interp;
axis ij image;
xlabel('r (\mum)');
ylabel('z (\mum)');
colorbarlabel = colorbar;
ylabel(colorbarlabel, '^2^0^8Pb (ppm)')
title('^2^0^8Pb');

subplot(2,3,5);
pcolor(r,z,U238);
colormap(parula);
colorbar;
shading interp;
axis ij image;
xlabel('r (\mum)');
ylabel('z (\mum)');
colorbarlabel = colorbar;
ylabel(colorbarlabel, '^2^3^8U (ppm)')
title('^2^3^8U');

subplot(2,3,6);
pcolor(r,z,Th232);
colormap(parula);
colorbar;
shading interp;
axis ij image;
xlabel('r (\mum)');
ylabel('z (\mum)');
colorbarlabel = colorbar;
ylabel(colorbarlabel, '^2^3^2Th (ppm)')
title('^2^3^2Th');

print('output_concentration_maps','-dpng');

%date maps
min=1000000;
max=0;
for j=1:1:Nr-1
    for i=2:1:Nz-1
        if(Mineral(i,j)==1 && Mineral(i,j+1)==1 && Mineral(i+1,j)==1 && Mineral(i-1,j)==1)
            if(min>agesPb206U238(i,j))
                min=agesPb206U238(i,j);
            end
            if(max<agesPb206U238(i,j))
                max=agesPb206U238(i,j);
            end
        end
    end
end
figure(1);clf;
fig=figure(1);
fig.Units = 'normalized';
fig.Position = [0 0 1 1];
subplot(2,2,1);
pcolor(r,z,agesPb206U238);
colormap(parula);
colorbar;
caxis([min*0.9 max*1.1]);
shading interp;
axis ij image;
xlabel('r (\mum)');
ylabel('z (\mum)');
colorbarlabel = colorbar;
ylabel(colorbarlabel, '^2^0^6Pb/^2^3^8U date (Ma)')
title('^2^0^6Pb/^2^3^8U date map');

min=1000000;
max=0;
for j=1:1:Nr-1
    for i=2:1:Nz-1
        if(Mineral(i,j)==1 && Mineral(i,j+1)==1 && Mineral(i+1,j)==1 && Mineral(i-1,j)==1)
            if(min>agesPb207U235(i,j))
                min=agesPb207U235(i,j);
            end
            if(max<agesPb207U235(i,j))
                max=agesPb207U235(i,j);
            end
        end
    end
end
subplot(2,2,2);
pcolor(r,z,agesPb207U235);
colormap(parula);
colorbar;
caxis([min*0.9 max*1.1]);
shading interp;
axis ij image;
xlabel('r (\mum)');
ylabel('z (\mum)');
colorbarlabel = colorbar;
ylabel(colorbarlabel, '^2^0^7Pb/^2^3^5U date (Ma)')
title('^2^0^7Pb/^2^3^5U date map');

min=1000000;
max=0;
for j=1:1:Nr-1
    for i=2:1:Nz-1
        if(Mineral(i,j)==1 && Mineral(i,j+1)==1 && Mineral(i+1,j)==1 && Mineral(i-1,j)==1)
            if(min>agesPb208Th232(i,j))
                min=agesPb208Th232(i,j);
            end
            if(max<agesPb208Th232(i,j))
                max=agesPb208Th232(i,j);
            end
        end
    end
end
subplot(2,2,3);
pcolor(r,z,agesPb208Th232);
colormap(parula);
colorbar;
caxis([min*0.9 max*1.1]);
shading interp;
axis ij image;
xlabel('r (\mum)');
ylabel('z (\mum)');
colorbarlabel = colorbar;
ylabel(colorbarlabel, '^2^0^8Pb/^2^3^2Th date (Ma)')
title('^2^0^8Pb/^2^3^2Th date map');

min=1000000;
max=0;
for j=1:1:Nr-1
    for i=2:1:Nz-1
        if(Mineral(i,j)==1 && Mineral(i,j+1)==1 && Mineral(i+1,j)==1 && Mineral(i-1,j)==1)
            if(min>agesPb207Pb206(i,j))
                min=agesPb207Pb206(i,j);
            end
            if(max<agesPb207Pb206(i,j))
                max=agesPb207Pb206(i,j);
            end
        end
    end
end
subplot(2,2,4);
pcolor(r,z,agesPb207Pb206);
colormap(parula);
colorbar;
caxis([min*0.9 max*1.1]);
shading interp;
axis ij image;
xlabel('r (\mum)');
ylabel('z (\mum)');
colorbarlabel = colorbar;
ylabel(colorbarlabel, '^2^0^7Pb-corrected (Ma)')
title('^2^0^7Pb-corrected date map');
clear max min;

print('output_date_maps','-dpng');